@@ -1,24 +1,13 @@
 --------------------------------------------------------------------------------
 module Language.Haskell.Stylish.Step
     ( Lines
-    , Module
     , Step (..)
     , makeStep
     ) where
 
 
 --------------------------------------------------------------------------------
-import qualified Language.Haskell.Exts as H
-
-
---------------------------------------------------------------------------------
-type Lines = [String]
-
-
---------------------------------------------------------------------------------
--- | Concrete module type
-type Module = (H.Module H.SrcSpanInfo, [H.Comment])
-
+import           Language.Haskell.Stylish.Module
 
 --------------------------------------------------------------------------------
 data Step = Step
@@ -26,7 +15,6 @@ data Step = Step
     , stepFilter :: Lines -> Module -> Lines
     }
 
-
 --------------------------------------------------------------------------------
 makeStep :: String -> (Lines -> Module -> Lines) -> Step
 makeStep = Step
