@@ -19,4 +19,9 @@ dropTrailingWhitespace = reverse . dropWhile isSpace . reverse
 
 --------------------------------------------------------------------------------
 step :: Step
-step = makeStep "TrailingWhitespace" $ \ls _ -> map dropTrailingWhitespace ls
+step = makeStep "TrailingWhitespace" $ \ls _ -> map dropTrailingWhitespace' ls
+  where
+    dropTrailingWhitespace' l = case l of
+      -- Preserve page breaks
+      "\12" -> l
+      _     -> dropTrailingWhitespace l
